/*
 * --COPYRIGHT--,BSD Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved. Redistribution and use in source and binary
 * forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must
 * retain the above copyright notice, this list of conditions and the
 * following disclaimer. * Redistributions in binary form must reproduce
 * the above copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with
 * the distribution. * Neither the name of Texas Instruments Incorporated 
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE. --/COPYRIGHT--
 */

/*******************************************************************************
 *
 *  utility.h - Definition file for utility functions
 *
 ******************************************************************************/

#ifndef UTILITY_H_
#define UTILITY_H_

/*****************************************************************************/

// TBREMOVED when debugging is complete
#define DEBUG_OF_CODE

// change to reflect firmware revision
#define FW_VERSION 0.5		// uses utility_v0p2.c
#define DeviceID   8436     // DRV8436
#define RDSON      900      // 900 mOhm
#define VM_Variant  24      // 24V max
#define IF_Variant   2      // 2 is PWM interface
// count to toggle status LED and force update
#define LED_TIMEOUT 6000

// 
#define	DEFAULT_IFS				0
#define	DEFAULT_SENSE_RESISTOR	1   // Just in case this is not removed
#define	DEFAULT_VREF_VAL		300
// #define V3P3_TIMES_GAIN         1.5	// 3.3 divide 2.2V/A, use constant
#define	DEFAULT_TORQUE			0	// 100% (use index value 0 
						// - 3)
#define DEFAULT_DECAY			2	// mixed decay


// IFS defines
// this are used to calculate the chopping current
#define V3P3_TIMES_GAIN			1.5	// 3.3 divide by 2.2V/A, use constant
						// constant
#define VREF_BIT				0.0009775	// 1/1023
								// per
								// step
#define TORQUE_STEP				0	// 1/4 per step not needed


// GPIO Port 1 Definitions
#define WAKE		BIT0	// P1.0



// GPIO Port 2 Definitions
#define M1_BIN2_PD	BIT0	// P2.0
#define ADECAY		BIT1	// P2.1
#define ADECAY_PD	BIT2	// P2.2
#define BDECAY		BIT3	// P2.3
#define BDECAY_PD	BIT4	// P2.4
#define TOFF		BIT5	// P2.5
#define TOFF_PD		BIT6	// P2.6
#define FAULT		BIT7	// P2.7
//#define TRQ1		BIT2	// P2.2  TBREMOVED
//#define TRQ0		BIT5	// P2.5  TBREMOVED


// #define PARA		BIT6	// P2.6 // TBREMOVED
// These are used to remove the ADC connection when Hi-Z
#define CAPD_M1_BIN2_PD	BIT0
#define CAPD_ADECAY 	BIT1
#define CAPD_ADECAY_PD 	BIT2
#define CAPD_BDECAY 	BIT3
#define CAPD_BDECAY_PD 	BIT4
#define CAPD_TOFF		BIT5
#define CAPD_TOFF_PD	BIT6

//#define CAPD_AIN1	BIT3	// TBREMOVED

//#define CAPD_TRQ1	BIT2  // TBREMOVED
//#define CAPD_TRQ0	BIT5  // TBREMOVED
//#define CAPD_PARA	BIT6	// TBREMOVED

// GPIO Port 4 Definitions
#define BIN2		BIT1	// P4.1
#define BIN1		BIT2	// P4.2
#define AIN1		BIT3	// P4.3
#define AIN2		BIT4	// P4.4


//#define V3P3_GPIO   BIT7	// P4.7  // TBREMOVED

// GPIO Port 6 Definitions, bitt 4 through 0 are used to identify devices
#define RDSON0		BIT0
#define RDSON1		BIT1
#define RDSON2		BIT2
#define INTERFACE	BIT3
#define VM_VOLTAGE	BIT4
#define AVREF   	BIT7	// P6.7 ***
#define BVREF		BIT6	// P6.6 ****BVREF
#define STATUS		BIT5	// P6.5
#define AVREF_DAC	DAC12_1DAT  // P6.7
#define BVREF_DAC	DAC12_0DAT  // P6.6

// 
// defines to make code more readable, and easier to fix
// 

#define toggle_status_led 		P6OUT ^=  STATUS

#define set_BDECAY_hi			P2OUT |=  BDECAY
#define set_BDECAY_lo			P2OUT &= ~BDECAY
#define set_BDECAY_PD_hi		P2OUT |=  BDECAY_PD
#define set_BDECAY_PD_lo		P2OUT &= ~BDECAY_PD
#define set_ADECAY_PD_hi		P2OUT |=  ADECAY_PD
#define set_ADECAY_PD_lo		P2OUT &= ~ADECAY_PD
#define set_ADECAY_hi			P2OUT |=  ADECAY
#define set_ADECAY_lo			P2OUT &= ~ADECAY

//#define set_DIR_hi				P4OUT |=  DIR // TBREMOVED
//#define set_DIR_lo				P4OUT &= ~DIR // TBREMOVED

//#define set_ENABLE_hi			P4OUT |=  ENABLE  // TBREMOVED
//#define set_ENABLE_lo			P4OUT &= ~ENABLE  // TBREMOVED

#define set_TOFF_hi				P2OUT |=  TOFF
#define set_TOFF_PD_hi			P2OUT |=  TOFF_PD
#define set_TOFF_lo				P2OUT &= ~TOFF
#define set_TOFF_PD_lo			P2OUT &= ~TOFF_PD

// TBREMOVED
//#define set_TRQ1_hi				P2OUT |=  TRQ1
//#define set_TRQ1_lo				P2OUT &= ~TRQ1
//#define set_TRQ0_hi				P2OUT |=  TRQ0
//#define set_TRQ0_lo				P2OUT &= ~TRQ0

#define set_AIN1_hi				P4OUT |=  AIN1
#define set_AIN1_lo				P4OUT &= ~AIN1
#define set_AIN2_hi				P4OUT |=  AIN2
#define set_AIN2_lo				P4OUT &= ~AIN2
#define set_BIN1_hi				P4OUT |=  BIN1
#define set_BIN1_lo				P4OUT &= ~BIN1
#define set_BIN2_hi				P4OUT |=  BIN2
#define set_BIN2_lo				P4OUT &= ~BIN2
#define set_M1_BIN2_PD_hi		P4OUT |=  M1_BIN2_PD
#define set_M1_BIN2_PD_lo		P4OUT &= ~M1_BIN2_PD


// TBREMOVED
//#define set_PARA_hi				P2OUT |=  PARA
//#define set_PARA_lo				P2OUT &= ~PARA

#define set_WAKE_hi				P1OUT |=  WAKE
#define set_WAKE_lo				P1OUT &= ~WAKE

// TBREMOVED
//#define enable_V3P3_GPIO		P4OUT |= V3P3_GPIO; P4DIR |=  V3P3_GPIO
//#define disable_V3P3_GPIO		P4DIR &= ~V3P3_GPIO

// enable/disable output
#define enable_ADECAY_output		P2DIR |=  ADECAY
#define disable_ADECAY_output		P2DIR &= ~ADECAY
#define enable_ADECAY_PD_output		P2DIR |=  ADECAY_PD
#define disable_ADECAY_PD_output	P2DIR &= ~ADECAY_PD
#define enable_BDECAY_output		P2DIR |=  BDECAY
#define disable_BDECAY_output		P2DIR &= ~BDECAY
#define enable_BDECAY_PD_output		P2DIR |=  BDECAY_PD
#define disable_BDECAY_PD_output	P2DIR &= ~BDECAY_PD

#define disable_ADECAY_input		CAPD  |=  CAPD_ADECAY
#define enable_ADECAY_input			CAPD  &= ~CAPD_ADECAY
#define disable_ADECAY_PD_input		CAPD  |=  CAPD_ADECAY_PD
#define enable_ADECAY_PD_input		CAPD  &= ~CAPD_ADECAY_PD
#define disable_BDECAY_input		CAPD  |=  CAPD_BDECAY
#define enable_BDECAY_input			CAPD  &= ~CAPD_BDECAY
#define disable_BDECAY_PD_input		CAPD  |=  CAPD_BDECAY_PD
#define enable_BDECAY_PD_input		CAPD  &= ~CAPD_BDECAY_PD

#define enable_TOFF_output			P2DIR |=  TOFF
#define disable_TOFF_output			P2DIR &= ~TOFF
#define enable_TOFF_PD_output		P2DIR |=  TOFF_PD
#define disable_TOFF_PD_output		P2DIR &= ~TOFF_PD

#define disable_TOFF_input			CAPD  |=  CAPD_TOFF
#define enable_TOFF_input			CAPD  &= ~CAPD_TOFF
#define disable_TOFF_PD_input		CAPD  |=  CAPD_TOFF_PD
#define enable_TOFF_PD_input		CAPD  &= ~CAPD_TOFF_PD


#define read_FAULT_pin			(P2IN & FAULT)

// TBREMOVED
//#define disable_ENABLE_output   P4DIR &= ~ENABLE
//#define enable_ENABLE_output    P4DIR |= ENABLE

// Custom Types
typedef enum { false, true } boolean;
typedef enum { low, high } gpio;

/*****************************************************************************/

// Declare Global Variables

// ***********************************************************************************
// Global Variables for GUI Composer *
// ***********************************************************************************
// Device Identification
extern unsigned int G_DeviceID;
extern unsigned int G_RDSON;
extern unsigned int G_VM_Variant;
extern unsigned int G_IF_Variant;
//
extern unsigned int G_PWM_AIN1;
extern unsigned int G_PWM_AIN2;
extern unsigned int G_PWM_BIN1;
extern unsigned int G_PWM_BIN2;

extern unsigned int G_Normal_A;
extern unsigned int G_Normal_B;
#ifdef DEBUG_OF_CODE
extern unsigned int G_Parallel; // TBREMOVED Keep during debug
#endif
extern unsigned int G_Stepper;

// GPIO
extern gpio     G_WAKE;		// Wake device 0 - Sleep 1 - Wake
extern gpio     G_DIRECTION;	// Direction 0 - Forward 1 - Reverse
extern gpio     G_FAULT;	// Fault Status 0 - Fault 1 - OK
extern gpio     G_RUN;

extern unsigned int G_Mode;

// PWM Generation

extern unsigned long G_AVREF_VAL;	// scale for current
extern unsigned long G_BVREF_VAL;	// scale for current
extern unsigned int G_PPS_Value;
extern unsigned int G_TORQUE;	// Torque (1 of 4)
extern unsigned int G_TOFF;	// PWM off time (1 of 3)

extern float    G_AVREF;	// VREF
extern float    G_AIFS;		// Full scale current
extern float    G_BVREF;	// VREF
extern float    G_BIFS;		// Full scale current
extern float    G_IRSENSE;	// sense value resistor
extern unsigned int G_ADECAY;	// ADECAY (1 of 3)
extern unsigned int G_BDECAY;	// BDECAY (1 of 3)
extern float    G_FIRMWARE_VERSION;	// keep track of firmware

float           vref_times_irsense_times_constant;
float           torque_multiplier;
float           avref_multiplier;
float           bvref_multiplier;
float           step_multiplier;
float           irsense_multiplier;

boolean         UpdateChoppingCurrentA;
boolean         UpdateChoppingCurrentB;

// use 0/1 and shift by this value when
// necessary
unsigned int    LED_TOGGLE_CNT;	// count to determine if LED should be
				// toggled

/*****************************************************************************/
// Function Declarations

// Main Functions
void            Initialize();
void            UpdateSystem();

// GPIO/GUI Functions
void            SetDefaultGUIVals();
void            SetDefaultGPIOVals();


#endif				/* UTILITY_H_ */
